| [**Home**](#) |

---

# Running Tests

## On local machine

Before running the tests, make sure that you are using appropriate chrome driver for the version of Chrome browser installed in your machine.
If your Chrome version is 105, then change the following attribute value to 105 in contour-app-test/ui-test/package.json.

    "chromedriver": "^105.0.0",

From contour-app-test/ui-test folder, run the following commands,

```
npm install
npx contour-fet -c './test/suite/<path to config file>'
```

For example,

```
npx contour-fet -c './test/suite/login.ts'
```

## On Selenoid Hub

[Selenoid](https://aerokube.com/selenoid/) is being used as Selenium Hub implementation in our infrastructure. Tests can be sent to Selenoid hub, in which case, selenoid handles the distribution of tests among multiple nodes (docker containers with specific browser & version installed) and also test execution.

To run the tests on Selenoid hub, from contour-app-test/ui-test folder, run the following command (using hub config file),

```s
npm install
npx contour-fet -c './test/suite/<path to hub config file>'
```

For example,

```
npx contour-fet -c './test/suite/login.hub.ts'
```

Test execution can be viewed in realtime [here](http://selenoid.dev.contournetwork.io:8080/#/).

# Change Management Checklist

1. Feature file is placed in appropriate folder
2. Feature file is added to appropriate test suite
3. Feature file is tagged with appropriate functional tag. For list of valid functional tags, refer this [secion](#test-case-functional-tags)
4. Test the changes by running the tests against Selenoid Hub (Chrome browser)
5. Once all above criterias are met, raise a PR to develop branch

# Notes

## Dependency issues:

- Can not upgrade @wdio/local-runner due to [this](https://github.com/webdriverio/webdriverio/issues/6316) issue. Using fixed version 7.0.5 as temporary solution.

# Appendix

## Test Case Functional Tags

| Function               | Tag                 |
| ---------------------- | ------------------- |
| DC Issuance            | @DCIssuance         |
| DC Amendment           | @DCAmendment        |
| DC Cancellation        | @DCCancellation     |
| DC Presentation        | @DCPresentation     |
| DC Templates           | @DCTemplates        |
| BNO                    | @BNO                |
| User Management        | @UserMgmt           |
| P2P Message            | @P2PMsg             |
| EssDocs                | @EssDocs            |
| Printing               | @Printing           |
| Absolute Session Limit | @SessionLimit       |
| Data Model             | @DataModel          |
| Network Map            | @NetworkMap         |
| MFA                    | @MFA                |
| SSO                    | @SSO                |
| Security               | @Security           |
| Customisation          | @Customisation      |
| Credit Guarantee       | @CreditGuarantee    |
| Localisation           | @Localisation       |
| Audit Trail            | @Audit Trail        |
| Email Notifications    | @EmailNotifications |
| Login                  | @Login              |
| Reset Password         | @ResetPassword      |
| Forgot Password        | @ForgotPassword     |
| E2E                    | @E2E                |
