import { Button, Input, Web } from '@contour/fet/lib/elements';

import { ContourBasePage } from '../../base/basePage';
import locator from './login.locator';

export class LoginPage extends ContourBasePage {
  constructor(url?: string) {
    super(locator, url);
  }

  getUsernameInput() {
    return new Input(this.locators.usernameInput);
  }

  getPasswordInput() {
    return new Input(this.locators.passwordInput);
  }

  getSubmitButton() {
    return new Button(this.locators.loginButton);
  }

  async login(loginName: string, loginPassword: string): Promise<void> {
    await this.getUsernameInput().setInputField(loginName);
    await this.getPasswordInput().setInputField(loginPassword);
    await this.getSubmitButton().clickElement();
  }

  async clearLoginForm(): Promise<void> {
    await this.getUsernameInput().clearInputField();
    await this.getPasswordInput().clearInputField();
  }

  async assertPage(): Promise<void> {
    await this.getSubmitButton().checkIsDisplayed();
    await Web.checkTitleContainsText(this.title);
  }
}
