import { DataTable, Given } from '@cucumber/cucumber';

import * as generateDcUtil from '../../util/generateDC';

/**
 * API
 */
Given(
  /^applicant created dc for scenario (sc1|sc2|sc2split|sc3split|sc4|sc5|sc6|sc8|sc10|sc11|sc12) proceeded by "([^"]+)?"$/,
  async function (scenario: string, proceedRoles: string) {
    await generateDcUtil.generateApplicantProceededDc(scenario, proceedRoles);
  }
);

Given(
  /^applicant created dc draft for beneficiary for scenario (sc1|sc2|sc2split|sc3split|sc4|sc5|sc6|sc8|sc10|sc11|sc12) proceeded by "([^"]+)?"$/,
  async function (scenario, proceedRoles) {
    await generateDcUtil.generateApplicantProceededDcForBeneficiary(scenario, proceedRoles);
  }
);

Given(
  /^applicant created dc for scenario (sc1|sc2|sc4|sc5|sc6|sc8|sc10|sc11) with (\d+) attachments proceeded by "([^"]+)?"$/,
  async function (scenario, numberOfAttachment, proceedRoles) {
    await generateDcUtil.generateApplicantProceededDc(scenario, proceedRoles, numberOfAttachment);
  }
);

Given(
  /^applicant created dc for scenario (sc1|sc2|sc4|sc5|sc6|sc8|sc10|sc11) with following document source proceeded by "([^"]+)?"$/,
  async function (scenario: string, proceedRoles: string, documentSourceDataTable: DataTable) {
    await generateDcUtil.generateApplicantProceededDc(scenario, proceedRoles, 0, documentSourceDataTable);
  }
);

Given(
  /^issuingBank (issued|rejected) dc for scenario (sc1|sc2|sc4|sc5|sc6|sc8|sc10|sc11) proceeded by "([^"]+)?"$/,
  async function (action, scenario, proceedRoles) {
    await generateDcUtil.generateIssuingBankProceededDc(scenario, action, proceedRoles);
  }
);

Given(
  /^issuingBank (issued|rejected) dc for scenario (sc1|sc2|sc4|sc5|sc6|sc8|sc10|sc11) with (\d+) attachments proceeded by "([^"]+)?"$/,
  async function (action, scenario, attachmentNum, proceedRoles) {
    await generateDcUtil.generateIssuingBankProceededDc(scenario, action, proceedRoles, attachmentNum);
  }
);

Given(
  /^issuingBank (issued|rejected) dc for scenario (sc1|sc2|sc4|sc5|sc6|sc8|sc10|sc11) with following document source proceeded by "([^"]+)?"$/,
  async function (action: string, scenario: string, proceedRoles: string, documentSourceDataTable: DataTable) {
    await generateDcUtil.generateIssuingBankProceededDc(scenario, action, proceedRoles, 0, documentSourceDataTable);
  }
);

Given(
  /^confirmingBank (advised with confirmation|advised without confirmation|rejected) dc for scenario (sc4|sc5|sc6|sc8|sc10|sc11) proceeded by "([^"]+)?"$/,
  async function (action, scenario, proceedRoles) {
    await generateDcUtil.generateConfirmingBankProceededDc(scenario, action, proceedRoles);
  }
);

Given(
  /^confirmingBank (advised with confirmation|advised without confirmation|rejected) dc for scenario (sc4|sc5|sc6|sc8|sc10|sc11) with (\d+) attachments proceeded by "([^"]+)?"$/,
  async function (action, scenario, attachmentNum, proceedRoles) {
    await generateDcUtil.generateConfirmingBankProceededDc(scenario, action, proceedRoles, attachmentNum);
  }
);

Given(
  /^confirmingBank (advised with confirmation|advised without confirmation|rejected) dc for scenario (sc4|sc5|sc6|sc8|sc10|sc11) with following document source proceeded by "([^"]+)?"$/,
  async function (action, scenario, proceedRoles, documentSourceDataTable) {
    await generateDcUtil.generateConfirmingBankProceededDc(scenario, action, proceedRoles, 0, documentSourceDataTable);
  }
);

Given(
  /^nominatedBank (advised|rejected) dc for scenario (sc2|sc8|sc11) proceeded by "([^"]+)?"$/,
  async function (action, scenario, proceedRoles) {
    await generateDcUtil.generateNominatedBankProceededDc(scenario, action, proceedRoles);
  }
);

Given(
  /^nominatedBank (advised|rejected) dc for scenario (sc2|sc8|sc11) with (\d+) attachments proceeded by "([^"]+)?"$/,
  async function (action, scenario, attachmentNum, proceedRoles) {
    await generateDcUtil.generateNominatedBankProceededDc(scenario, action, proceedRoles, attachmentNum);
  }
);

Given(
  /^nominatedBank (advised|rejected) dc for scenario (sc2|sc8|sc11) with following document source proceeded by "([^"]+)?"$/,
  async function (action, scenario, proceedRoles, documentSourceDataTable) {
    await generateDcUtil.generateNominatedBankProceededDc(scenario, action, proceedRoles, 0, documentSourceDataTable);
  }
);

Given(
  /^advisingBank (advised|rejected) dc for scenario (sc1|sc2|sc4|sc5|sc6|sc8|sc10|sc11|sc12) proceeded by "([^"]+)?"$/,
  async function (action, scenario, proceedRoles) {
    await generateDcUtil.generateAdvisingBankProceededDc(scenario, action, proceedRoles);
  }
);

Given(
  /^advisingBank (advised|rejected) dc for scenario (sc1|sc2|sc4|sc5|sc6|sc8|sc10|sc11) with (\d+) attachments proceeded by "([^"]+)?"$/,
  async function (action, scenario, attachmentNum, proceedRoles) {
    await generateDcUtil.generateAdvisingBankProceededDc(scenario, action, proceedRoles, attachmentNum);
  }
);

Given(
  /^advisingBank (advised|rejected) dc for scenario (sc1|sc2|sc4|sc5|sc6|sc8|sc10|sc11) with following document source proceeded by "([^"]+)?"$/,
  async function (action, scenario, proceedRoles, documentSourceDataTable) {
    await generateDcUtil.generateAdvisingBankProceededDc(scenario, action, proceedRoles, 0, documentSourceDataTable);
  }
);
Given(
  /^issuing bank (issued|rejected) dc for scenario (sc1|sc2|sc4|sc5|sc6|sc8|sc10|sc11)$/,
  async function (action, scenario) {
    await generateDcUtil.generateIssuingBankProceededDc(scenario, action);
  }
);

Given(
  /^confirming bank (advised with confirmation|advised without confirmation|rejected) dc for scenario (sc4|sc5|sc6|sc8|sc10|sc11)$/,
  async function (action, scenario) {
    await generateDcUtil.generateConfirmingBankProceededDc(scenario, action);
  }
);

Given(/^nominated bank (advised|rejected) dc for scenario (sc2|sc8|sc11|sc12)$/, async function (action, scenario) {
  await generateDcUtil.generateNominatedBankProceededDc(scenario, action);
});

Given(
  /^advising bank (advised|rejected) dc for scenario (sc1|sc2|sc4|sc5|sc6|sc8|sc10|sc11)$/,
  async function (action, scenario) {
    await generateDcUtil.generateAdvisingBankProceededDc(scenario, action);
  }
);
